/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Adblock Plus.
 *
 * The Initial Developer of the Original Code is
 * Wladimir Palant.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

/*
 * Utility functions and classes.
 * This file is included from nsAdblockPlus.js.
 */

// A hash table class - sort of
function HashTable() {
	this.data = {};
}
HashTable.prototype = {
	get: function(key) {
		key = " " + key;
		if (key in this.data)
			return this.data[key];
		else
			return undefined;
	},
	put: function(key, value) {
		key = " " + key;
		this.data[key] = value;
	},
	remove: function(key) {
		key = " " + key;
		delete this.data[key];
	},
	has: function(key) {
		key = " " + key;
		return (key in this.data);
	},
	clear: function() {
		this.data = {};
	},
	keys: function() {
		var result = [];
		for (var key in this.data)
			if (key.indexOf(" ") == 0)
				result.push(key.substr(1));

		return result;
	},
	values: function() {
		var result = [];
		for (var key in this.data)
			if (key.indexOf(" ") == 0)
				result.push(this.data[key]);

		return result;
	}
}
abp.HashTable = HashTable;

// String service
var stringService = Components.classes["@mozilla.org/intl/stringbundle;1"]
															.getService(Components.interfaces.nsIStringBundleService);
var strings = stringService.createBundle("chrome://adblockplus/locale/global.properties");
abp.getString = function(name) {
	return strings.GetStringFromName(name);
};

// Retrieves the window object for a node or returns null if it isn't possible
function getWindow(insecNode) {
	if (secureGet(insecNode, "nodeType") != Node.DOCUMENT_NODE)
		insecNode = secureGet(insecNode, "ownerDocument");

	if (!insecNode || secureGet(insecNode, "nodeType") != Node.DOCUMENT_NODE)
		return null;

	return secureGet(insecNode, "defaultView");
}

// Retrieves the main window object for a node or returns null if it isn't possible
function getTopWindow(insecNode) {
	if (secureGet(insecNode, "nodeType") != Node.DOCUMENT_NODE)
		insecNode = secureGet(insecNode, "ownerDocument");

	if (!insecNode || secureGet(insecNode, "nodeType") != Node.DOCUMENT_NODE)
		return null;

	return secureGet(insecNode, "defaultView", "top");
}

// hides a blocked element and collapses it if necessary
function hideNode(insecNode, insecWnd, collapse) {
	// hide object tab
	var insecTab = secureGet(insecNode, "nextSibling");
	if (insecTab && secureGet(insecTab, "nodeType") == Node.ELEMENT_NODE &&
			secureLookup(insecTab, "hasAttribute")("AdblockTab"))
		secureSet(insecTab, "style", "display", "none");

	// special handling for applets -- disable by specifying the Applet base class
	var nodeName = secureGet(insecNode, "nodeName");
	if (nodeName && nodeName.toLowerCase() == "applet")
		secureLookup(insecNode, "setAttribute")("code", "java.applet.Applet");

	// Empty frames to avoid a graphics glitch -- old api only
	if (oldStyleAPI && insecNode instanceof Window) {
		var insecRoot = secureGet(insecNode, "document", "documentElement");
		var removeFunc = secureLookup(insecRoot, "removeChild");
		var insecChild;
		while ((insecChild = secureGet(insecRoot, "firstChild")) != null)
			removeFunc(insecChild);
	}

	if (collapse) {
		if (insecNode instanceof Window)
			insecNode = secureGet(insecNode, "frameElement");

		// adjust frameset's cols/rows for frames
		var attrFunc = secureLookup(insecNode, "parentNode", "hasAttribute");
		var parentNode = secureGet(insecNode, "parentNode");
		if (attrFunc && secureGet(parentNode, "nodeName").toLowerCase() == "frameset" &&
				(attrFunc("cols") ^ attrFunc("rows"))) {
			var frameTags = {FRAME: true, FRAMESET: true};
			var index = -1;
			for (var insecFrame = insecNode; insecFrame; insecFrame = secureGet(insecFrame, "previousSibling"))
				if (secureGet(insecFrame, "nodeName").toUpperCase() in frameTags)
					index++;
	
			var attr = (attrFunc("rows") ? "rows" : "cols");
			secureLookup(insecWnd, "setTimeout")(hideFrameCallback, 0, parentNode, attr, index);
		}
		else
			secureLookup(insecWnd, "setTimeout")(hideCallback, 0, insecNode);
	}
}

function hideCallback(insecNode) {
	secureSet(insecNode, "style", "display", "none");
}

function hideFrameCallback(insecFrameset, attr, index) {
	var weights = secureLookup(insecFrameset, "getAttribute")(attr).split(",");
	weights[index] = "0";
	secureLookup(insecFrameset, "setAttribute")(attr, weights.join(","));
}

// Creates a tab above/below the new object node
function addObjectTab(insecNode, location, insecWnd) {
	// Prevent readding tabs to elements that already have one
	if (secureGet(insecNode, "nextSibling", "nodeType") == Node.ELEMENT_NODE &&
			secureLookup(insecNode, "nextSibling", "hasAttribute")("AdblockTab"))
		return;

	// Tab dimensions
	var tabWidth = 70;
	var tabHeight = 18;

	// Decide whether to display the tab on top or the bottom of the object
	var offsetTop = 0;
	for (var insecOffsetNode = insecNode; insecOffsetNode; insecOffsetNode = secureGet(insecOffsetNode, "offsetParent"))
		offsetTop += secureGet(insecOffsetNode, "offsetTop");

	var onTop = (offsetTop > 40);

	// Compose tab
	var insecDoc = secureGet(insecNode, "ownerDocument");
	if (!insecDoc)
		return;

	var label = secureLookup(insecDoc, "createElement")("div");
	label.appendChild(secureLookup(insecDoc, "createTextNode")("Adblock"));
	label.style.display = "block";
	label.style.position = "relative";
	label.style.left = -tabWidth + "px";
	label.style.top = (onTop ? -tabHeight + "px" :  "0px");
	label.style.width = (tabWidth - 4) + "px";
	label.style.height = (tabHeight - 2) + "px";
	label.style.borderStyle = "ridge";
	label.style.borderWidth = (onTop ? "2px 2px 0px 2px" : "0px 2px 2px 2px");
	label.style.MozBorderRadiusTopleft = label.style.MozBorderRadiusTopright = (onTop ? "10px" : "0px");
	label.style.MozBorderRadiusBottomleft = label.style.MozBorderRadiusBottomright = (onTop ? "0px" : "10px");
	label.style.backgroundColor = "white";
	label.style.color = "black";
	label.style.cursor = "pointer";
	label.style.fontFamily = "Arial,Helvetica,Sans-serif";
	label.style.fontSize = "12px";
	label.style.fontStyle = "normal";
	label.style.fontVariant = "normal";
	label.style.fontWeight = "normal";
	label.style.letterSpacing = "normal";
	label.style.lineHeight = "normal";
	label.style.textAlign = "center";
	label.style.textDecoration = "none";
	label.style.textIndent = "0px";
	label.style.textTransform = "none";
	label.style.direction = "ltr";

	var tab = secureLookup(insecDoc, "createElement")("div");
	tab.appendChild(label);
	tab.style.display = "block";
	tab.style.position = "relative"
	tab.style.overflow = "visible";
	tab.style.width = "0px";
	tab.style.height = "0px";
	tab.style.left = "0px";
	tab.style.paddingLeft = secureGet(insecNode, "offsetWidth") + "px";
	tab.style.top = (onTop ? -secureGet(insecNode, "offsetHeight") + "px" : "0px");
	tab.style.zIndex = 65535;
	tab.style.MozOpacity = "0.5";

	// Prevent object tab from being added multiple times
	tab.setAttribute("AdblockTab", "true");
	
	// Click event handler
	label.addEventListener("click", function() {
		abp.openSettingsDialog(insecWnd, location);
	}, false);

	// Insert tab into the document
	var nextSibling = secureGet(insecNode, "nextSibling");
	if (nextSibling)
		secureLookup(insecNode, "parentNode", "insertBefore")(tab, nextSibling);
	else
		secureLookup(insecNode, "parentNode", "appendChild")(tab);
}

// Sets a timeout, compatible with both nsITimer and nsIScriptableTimer
function createTimer(callback, delay) {
	var timer = Components.classes["@mozilla.org/timer;1"];
	timer = timer.createInstance(Components.interfaces.nsITimer);
	timer.init({observe: callback}, delay, timer.TYPE_ONE_SHOT);
	return timer;
}

